import 'dart:async';

import 'package:flutter/widgets.dart';
import 'package:get/get.dart';
import 'package:fluttertoast/fluttertoast.dart';
import '../utils/code_send_enum.dart';

class CLXCodeManageLogic extends GetxController {
  CLXCodeManageLogic(
      {this.maxLength = 6,
      this.isFirstResponsed = false,
      this.isAccessOutbound = true});

  /// 验证码长度
  final int maxLength;

  /// 是否是成为第一响应者
  final bool isFirstResponsed;

  /// 是否接入外呼功能
  final bool isAccessOutbound;

  /// 验证码输入框焦点
  final FocusNode codeInoutFocus = FocusNode();

  /// 验证码输入框控制器
  final TextEditingController codeInoutCtrl = TextEditingController();

  /// 倒计时
  RxInt countdown = 60.obs;

  /// 定时器
  Timer? timer;

  /// 获取短信验证码
  Future<bool?> Function()? getSmsVerificationCode;

  /// 获取语音验证码
  Future<bool?> Function()? obtainVoiceVerificationCode;

  /// 校验验证码输入是否有误
  Future<String?> Function(String code)? verifyCode;

  /// 联系客服点击事件
  Future Function()? contactCustomerService;

  /// 验证码校验错误信息
  RxString verifyCodeError = ''.obs;

  /// 获取验证码按钮文案
  RxString codeSendBtn = '获取验证码'.obs;

  /// 验证码发送状态
  Rx<CodeSendType> codeSendType = CodeSendType.none.obs;

  /// 是否展示提示信息模块
  RxBool isShowTip = false.obs;

  /// 刷新页面需要
  RxString refreshPage = ''.obs;

  /// 是否允许发送验证码
  bool canSend = true;

  @override
  void onInit() {
    super.onInit();

    codeInoutCtrl.addListener(_verifyCodeInput);

    if (isFirstResponsed) {
      codeInoutFocus.requestFocus();
    }
  }

  /// 校验验证码
  _verifyCodeInput() async {
    /// 处理多次发送校验间隔问题
    if (refreshPage.value == codeInoutCtrl.text) {
      return;
    }
    handleTipInfo();
    update();
    refreshPage.value = codeInoutCtrl.text;
    if (codeInoutCtrl.text.length != maxLength) {
      return;
    }
    final result = await verifyCode?.call(codeInoutCtrl.text) ?? '';
    verifyCodeError.value = result;
    handleTipInfo();
  }

  /// 外部调用开启定时器
  startTimerAndChangeType() {
    if (!canSend) {
      return;
    }
    canSend = false;
    _startCountdown();
    handleCodeSendType();
  }

  /// 发送短信验证码
  sendSmsCode() async {
    if (!canSend) {
      return;
    }
    canSend = false;
    if (countdown.value != 60) {
      return;
    }
    final result = await getSmsVerificationCode?.call();

    if (result == true) {
      handleCodeSendType();
      _startCountdown();
    } else {
      canSend = true;
    }
  }

  /// 发送语音验证码
  sendVoiceCode() async {
    if (!canSend) {
      return;
    }
    canSend = false;
    if (countdown.value != 60) {
      return;
    }
    final result = await obtainVoiceVerificationCode?.call();

    if (result == true) {
      handleCodeSendType(isSmsSend: false);
      _startCountdown(isSmsSend: false);
      Fluttertoast.showToast(
        msg: '即将向您电话播报',
        toastLength: Toast.LENGTH_SHORT,
        gravity: ToastGravity.CENTER,
        timeInSecForIosWeb: 2,
      );
    } else {
      canSend = true;
    }
  }

  /// 联系客服
  contactService() async {
    await contactCustomerService?.call();
  }

  /// 开启倒计时
  _startCountdown({bool isSmsSend = true}) {
    if (countdown.value != 60) {
      return;
    }
    timer = Timer.periodic(const Duration(seconds: 1), (timer) {
      if (countdown.value == 0) {
        timer.cancel();
        codeSendBtn.value = '';
        countdown.value = 60;
        canSend = true;
        handleCodeSendType(isSmsSend: isSmsSend);
        handleTipInfo();
      } else {
        countdown.value--;
        codeSendBtn.value = '已发送${countdown.value}s';
      }
    });
  }

  /// 处理验证码发送状态
  handleCodeSendType({bool isSmsSend = true}) {
    switch (codeSendType.value) {
      case CodeSendType.none:
        codeSendType.value = CodeSendType.firstSendSmsCode;
        break;
      case CodeSendType.firstSendSmsCode:
        codeSendType.value = CodeSendType.notReceiveSmsCode;
        break;
      case CodeSendType.notReceiveSmsCode:
        if (isSmsSend) {
          codeSendType.value = CodeSendType.resendSmsCode;
        } else {
          codeSendType.value = CodeSendType.firstSendVoiceCode;
        }
        break;
      case CodeSendType.resendSmsCode:
        if (isAccessOutbound) {
          codeSendType.value = CodeSendType.notReceiveResendSmsCode;
        }
        break;
      case CodeSendType.notReceiveResendSmsCode:
        codeSendType.value = CodeSendType.firstSendVoiceCode;
        break;
      case CodeSendType.firstSendVoiceCode:
        codeSendType.value = CodeSendType.notReceiveVoiceCode;
        break;
      case CodeSendType.notReceiveVoiceCode:
        codeSendType.value = CodeSendType.resendVoiceCode;
        break;
      case CodeSendType.resendVoiceCode:
        codeSendType.value = CodeSendType.notReceiveResendVoiceCode;
        break;
      case CodeSendType.notReceiveResendVoiceCode:
        codeSendType.value = CodeSendType.resendVoiceCode;
        break;
      default:
        codeSendType.value = CodeSendType.notReceiveResendVoiceCode;
        break;
    }
  }

  /// 处理提示信息
  handleTipInfo() {
    /// 如果输入框内容不为空，则不展示提示信息
    isShowTip.value = verifyCodeError.value.isNotEmpty ||
        codeInoutCtrl.text.isEmpty && countdown.value == 60;
  }
}
